/* eslint-disable @typescript-eslint/no-explicit-any */
import SlackHook from 'winston-slack-webhook-transport';

import { LOG_LEVEL } from '#/logger/logger.types';

import { AttachementField, Attachment, ErrorMessageWithAttachment } from './slack.transport.types';

export const getMessageForSlack = (info: any, nodeEnv: string): ErrorMessageWithAttachment | object => {
	const errorObject = {
		context: info.context ?? info[Symbol.for('message')].context,
		message: info.message ?? undefined,
		level: (info.level as string) ?? (info[Symbol.for('level')] as string),
		splat: info[Symbol.for('splat')] ?? undefined,
		data: info.data ?? info[Symbol.for('message')].data,
		timestamp: (info.timestamp as string) ?? (info[Symbol.for('message')].timestamp as string),
		symbolMessage: JSON.parse(info[Symbol.for('message')]) ?? undefined,
	};

	const isAxiosError = JSON.stringify(errorObject.symbolMessage.data).includes('AxiosError');
	const attachments: Attachment[] = [];
	const fields: AttachementField[] = [];

	if (!isAxiosError) {
		for (const [key, value] of Object.entries(errorObject.symbolMessage.data)) {
			fields.push({
				title: `${key}:`,
				value: JSON.stringify(value, null, '\t'),
				short: false,
			});
		}
	} else {
		fields.push({
			title: `AxiosError:`,
			value: errorObject.symbolMessage.data.stack,
			short: false,
		});
	}

	attachments.push({
		// green
		color: '#36a64f',
		fields,
		footer: `*${errorObject.timestamp} - ${nodeEnv}*`.toUpperCase(),
	});

	const slackMessage: ErrorMessageWithAttachment = {
		attachments,
		blocks: [
			{
				type: 'header',
				text: {
					type: 'plain_text',
					text: ':x:The service got an error :x:',
					emoji: true,
				},
			},
			{
				type: 'context',
				elements: [
					{
						type: 'mrkdwn',
						text: `:exclamation:Error level: *${errorObject.level}*`.toUpperCase() + `:exclamation:`,
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Context:*',
					},
					{
						type: 'mrkdwn',
						text: errorObject.context,
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Response:*',
					},
					{
						type: 'mrkdwn',
						text: !isAxiosError ? JSON.stringify(errorObject.data, null, '\t') : JSON.stringify(errorObject.data.message, null, '\t'),
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Message:*',
					},
					{
						type: 'mrkdwn',
						text: errorObject.message,
					},
				],
			},
		],
	};

	return slackMessage;
};

export default class SlackTransport {
	public static create(nodeEnv: string, webhookUrl: string) {
		return new SlackHook({
			level: LOG_LEVEL.Error || LOG_LEVEL.Fatal || LOG_LEVEL.Emergency,
			webhookUrl,
			formatter: (info: any) => getMessageForSlack(info, nodeEnv),
		});
	}
}
